@@ -15,6 +15,7 @@ from .dreg import RegContainer, RegContainerReader, RegisterArray
 
 
 from .common import DApi2Side
+from . import _version 
 from enum import IntEnum
 from .common import dateToWord, versionToWord
 from .derror import DApiCommandError
@@ -47,6 +48,10 @@ class DApi2(object):
         self.comm = comm
         self.dev_mode = dev_mode
         
+    def version(self):
+        '''Returns the DAPI2 version'''
+        return _version.__version__
+        
         
     def sendMessage(self, msg):
         '''Sends a message to the `SLAVE` and waits its response.
@@ -398,11 +403,16 @@ class DApi2Commands(MetaDApi2Commands):
          
     
     def factSetSysinfo(self, sn, date, hv):    
+        date = dateToWord(date)
+        hv = versionToWord(*hv)
         msg = dmsg.CommandMessage(DApi2Commands.FACT_SET_SYSINFO)
         msg.setWord(sn)
-        msg.setWord(dateToWord(date))
-        msg.setWord(versionToWord(*hv))
+        msg.setWord(date)
+        msg.setWord(hv)
         msg = self._sendCmd(msg)
+        self.owner.regs.snr.value = sn
+        self.owner.regs.fdr.value = date
+        self.owner.regs.hvr.value = hv
          
 
     def factSetSrvinfo(self, service, date, tag):    
