@@ -12,7 +12,8 @@ from dboard.workspace import WorkspacesContainer, StandbyWorkspace, Workspace,\
     BaseWorkspace
 from dapi2.common import versionToWord, dateToWord, wordToDate, wordToVersion
 from dapi2.dreg.group import RegGroup
-from dboard.common import LastReset, ValueRange, DBoardException
+from dboard.common import LastReset, ValueRange, DBoardException,\
+    SystemModeConfig
 import os
 from dapi2.dmsg.message import CommandMessage
 
@@ -27,6 +28,10 @@ class BaseDBoard(object):
     '''
     
     number = None
+    '''Board type number (str)'''
+     
+    wait_after_reprogramming = 8.0 #[s]
+    '''Timeout after programming to reconnect to the board expressed in second.'''
     
     # @classmethod
     # def allSubclasses(cls):
@@ -35,11 +40,19 @@ class BaseDBoard(object):
     
     @classmethod
     def getBoardClasses(cls):
-        '''Returns all known card classes'''
+        '''Returns all known board classes
+        
+        :return: All known board classes
+        :rtype: dict 
+        '''
         return dict([(c.number,c) for c in cls.__subclasses__()])
     
     @classmethod
     def getName(cls):
+        '''Returns the board type name
+        :return: The board type name
+        :rtype: str
+        '''
         return 'MB-'+cls.number
 
     def __init__(self, dapi, dmode=DBoardPreferedDapiMode.REGISTER):
@@ -59,6 +72,10 @@ class BaseDBoard(object):
         return self.getName()
         
     def initialize(self):
+        '''Initialize the board object.
+        
+        The connection with the hardware must be established.
+        '''
         self.log.debug('initialize')
         self.getRegisters('header')
         self.dapi.readRegs(self.regs.pcr)
@@ -94,7 +111,8 @@ class BaseDBoard(object):
         The writing will be done the next time the :meth:`DApi2.syncRegs <dapi2.dapi2.DApi2.syncRegs>` method is called.
         If it is synchronous, the modification is directly written in the card registers.
         
-        :param Register reg: The register to be modified
+        :param reg: The register to be modified
+        :type reg: Register, str
         :param int value: The new value
         :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
         '''
@@ -104,10 +122,19 @@ class BaseDBoard(object):
         if synchronous:
             self.dapi.syncRegs()
             #self.dapi.writeReg(reg)
-            
-    
+   
             
     def setBit(self, bit, value=1, synchronous=True):
+        '''Sets a value to a register's bit.
+        
+        .. seealso::
+            :meth:`setReg`
+        
+        :param bit: The register's bit to be modified
+        :type bit: RegBit, str
+        :param int value: The new value (0 or 1). Default 1.
+        :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
+        '''
         if not isinstance(bit, RegBit):
             bit = self.dapi.getBit(bit)
         v = bit.set(value) 
@@ -116,9 +143,27 @@ class BaseDBoard(object):
             self.dapi.syncRegs()
             
     def clearBit(self, bit, synchronous=True):
+        '''Clear the registers's bit.
+        
+        .. seealso::
+            :meth:`setReg`
+
+        :param bit: The register's bit to be modified
+        :type bit: RegBit, str
+        :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
+        '''
         self.setBit(bit, 0, synchronous)
         
     def toggleBit(self, bit, synchronous=True):
+        ''' Toggles the registers's bit.
+        
+        .. seealso::
+            :meth:`setReg`
+
+        :param bit: The register's bit to be modified
+        :type bit: RegBit, str
+        :param bool synchronous: Specifies whether the modification is synchronous (default) or not.
+        '''
         if not isinstance(bit, RegBit):
             bit = self.dapi.getBit(bit)
         bit.parent.alter( bit.toggle() )
@@ -195,6 +240,12 @@ class BaseDBoard(object):
         
         
     def setWorkspace(self, workspace):
+        '''Sets a new workspace.
+        
+        :param Workspace workspace: The new workspace to activate.
+        :type workspace: BaseWorkspace, int 
+        ''' 
+        
         if not isinstance(workspace, BaseWorkspace):
             workspace = self.workspaces.getByPAR(workspace)
         
@@ -210,18 +261,32 @@ class BaseDBoard(object):
         #self._workspace = workspace
             
     def getWorkspace(self, refresh=False):
+        '''Returns the current workspace.
+        
+        :param bool refresh: specifies whether to update the in-memory register with the one on the board.
+        ''' 
         if refresh:
             self.dapi.readReg(self.regs.par)
         #self._workspace = self.workspaces.getByPAR(self.regs.par.value)
         return self._workspace
             
     def isOnStandby(self):
+        '''Check if the board is on standby state.
+        
+        :return: True, if the board is on standby state ; False otherwise.
+        :rtype: bool
+        ''' 
         if self._workspace is None:
             self.getWorkspace(refresh=True)
         return self._workspace.standby 
     
     
     def hasBlueLight(self):
+        '''Check if the board has the blue light feature.
+        
+        :return: True, if the board has the blue light feature ; False otherwise.
+        :rtype: bool
+        ''' 
         try: 
             #self.log.debug('ssr2='+self.regs.ssr2.toString(depth=2))
             return self.regs.ssr2.alt_lmp.value == 1
@@ -230,11 +295,20 @@ class BaseDBoard(object):
             return self.regs.ssr2.alt_lmp.value == 1
         
     def getError(self, refresh=False):
+        '''Returns the board error code.
+      
+        :param bool refresh: Specifies whether to update the in-memory register with the one on the board.
+        
+        :return: Zero, if the boatrd is'nt in error or warning state : Non-zero, otherwise 
+        :rtype: int
+        ''' 
+        
         if refresh:
             self.dapi.readReg(self.regs.wer)
         return self.regs.wer.value
         
     def getMicrocontrollerID(self):
+        
         ret = bytearray(self.dapi.cmd.getMcId(0))
         ret.extend(self.dapi.cmd.getMcId(1))
         return ret
@@ -244,6 +318,18 @@ class BaseDBoard(object):
             self.dapi.readReg(self.regs.scsr)
         return LastReset(self.regs.scsr.reset.value)
     
+    def getSystemModeConfiguration(self, refresh=False):
+        if refresh:
+            self.dapi.readReg(self.regs.smr)
+        try:
+            return SystemModeConfig(self.regs.smr.value)
+        except DRegUndefinedError:
+            self.dapi.readReg(self.regs.smr)
+            return SystemModeConfig(self.regs.smr.value)
+        
+    def setSystemModeConfiguration(self, flags):
+        self.setReg(self.regs.smr, flags.value)
+    
     def getAccessLevel(self, refresh=False):
         if refresh or self.regs.scsr.isUndefined():
             self.dapi.readReg(self.regs.scsr)
@@ -262,13 +348,13 @@ class BaseDBoard(object):
         
     def getSpeedReference(self, refresh=False):
         if refresh: 
-            self.dapi.readReg(self.regs.ref)
-        return self.regs.ref.value        
+            self.dapi.readReg(self.regs.elcr)
+        return self.regs.elcr.value        
 
     def getPressure(self, refresh=False):
         if refresh: 
-            self.dapi.readReg(self.regs.cap)
-        return self.regs.cap.value        
+            self.dapi.readReg(self.regs.prcr)
+        return self.regs.prcr.value        
             
     def getFirmwareVersion(self):
         try:
@@ -517,7 +603,7 @@ class BaseDBoard(object):
         
         :param int sn: Serial number.
         :param Date fd: Factory date.
-        :param tuple hv: 2-tuple of intetger for major and minor version number.
+        :param tuple hv: 2-tuple of integer for major and minor version number.
         ''' 
         self.log.info('Set factory data. SN:{0:04d}, FD:{1:s}, HV:{2:d}.{3:02d}'.format(sn, fd.isoformat(), hv[0], hv[1]) )
         
@@ -527,12 +613,20 @@ class BaseDBoard(object):
             self.setRegisters(snr=sn, fdr=dateToWord(fd), hvr = versionToWord(*hv), synchronous=True)        
                
     def getFactoryData(self, read=True):
+        '''Returns the factory data.
+        
+        :return: A 3-tuple containing the serial number (int), the factory date (date) and the hardware version (2-tuple of int). 
+        '''
         if read:
             self.getRegisters('snr','fdr','hvr')
         
         return (self.regs.snr.value, wordToDate(self.regs.fdr.value), wordToVersion(self.regs.hvr.value))
         
         
+    def refreshAll(self):
+        self.refreshState()
+        self.refreshSetpoints()
+        
     def refreshState(self):
         #self.log.debug('refreshState')
         self.getRegisters('header', refresh=True)
@@ -551,7 +645,7 @@ class BaseDBoard(object):
                 return 0
             else:
                 return ord(c)
-
+        self.log.info('Firmware programming has started....')
         with open( firm.filename, 'rb' ) as f:
             f.seek(0, os.SEEK_END)
             size = f.tell()
@@ -562,12 +656,13 @@ class BaseDBoard(object):
             while i < size:
                 self.dapi.cmd.flashData([_readByte(x) for x in range(CommandMessage.MAX_DATA_SIZE)])
                 i+=CommandMessage.MAX_DATA_SIZE
-                #self.log.debug('Programming block {0} / {1}'.format(i//CommandMessage.MAX_DATA_SIZE,size//CommandMessage.MAX_DATA_SIZE))
+                self.log.debug('Programming block {0} / {1}'.format(i//CommandMessage.MAX_DATA_SIZE,size//CommandMessage.MAX_DATA_SIZE))
                 if callback is not None:
                     callback(i, size)
                     
 
         self.dapi.cmd.flashEnd()
+        self.log.info('Firmware programming is complete!')
         
         
     def onPARChanged(self, reg, old, value):
@@ -581,10 +676,19 @@ class BaseDBoard(object):
     @property
     def workspaces(self):
         return self._workspaces
+    @property
+    def name(self):
+        return self.getName().lower()
     
     
 class DBoard(BaseDBoard):
+    '''Generic class board.
+    
+    This class is used when the type of card could not be determined.
+    ''' 
     number = '00'
+    '''Board type number'''
+    
     pass
         
     
\ No newline at end of file
